@@ -16,6 +16,21 @@
 #include "space.hh"
 #include "translate.hh"
 
+AttributeId ATTRIB_BASE = AttributeId("base",89);
+AttributeId ATTRIB_DEADCODEDELAY = AttributeId("deadcodedelay",90);
+AttributeId ATTRIB_DELAY = AttributeId("delay", 91);
+AttributeId ATTRIB_LOGICALSIZE = AttributeId("logicalsize",92);
+AttributeId ATTRIB_PHYSICAL = AttributeId("physical",93);
+AttributeId ATTRIB_PIECE1 = AttributeId("piece1",94);	// piece attributes must have sequential ids
+AttributeId ATTRIB_PIECE2 = AttributeId("piece2",95);
+AttributeId ATTRIB_PIECE3 = AttributeId("piece3",96);
+AttributeId ATTRIB_PIECE4 = AttributeId("piece4",97);
+AttributeId ATTRIB_PIECE5 = AttributeId("piece5",98);
+AttributeId ATTRIB_PIECE6 = AttributeId("piece6",99);
+AttributeId ATTRIB_PIECE7 = AttributeId("piece7",100);
+AttributeId ATTRIB_PIECE8 = AttributeId("piece8",101);
+AttributeId ATTRIB_PIECE9 = AttributeId("piece9",102);
+
 /// Calculate \e highest based on \e addressSize, and \e wordsize.
 /// This also calculates the default pointerLowerBound
 void AddrSpace::calcScaleMask(void)
@@ -111,58 +126,51 @@ void AddrSpace::truncateSpace(uint4 newsize)
   calcScaleMask();
 }
 
-/// Write the main XML attributes for an address within this space
+/// Write the main attributes for an address within \b this space.
 /// The caller provides only the \e offset, and this routine fills
 /// in other details pertaining to this particular space.
-/// \param s is the stream to write to
+/// \param encoder is the stream encoder
 /// \param offset is the offset of the address
-void AddrSpace::saveXmlAttributes(ostream &s,uintb offset) const
+void AddrSpace::encodeAttributes(Encoder &encoder,uintb offset) const
 
 {
-  a_v(s,"space",getName());	// Just append the proper attributes
-  s << ' ' << "offset=\"";
-  printOffset(s,offset);
-  s << "\"";
+  encoder.writeSpace(ATTRIB_SPACE,this);
+  encoder.writeUnsignedInteger(ATTRIB_OFFSET, offset);
 }
 
-/// Write the main XML attributes of an address with this space
+/// Write the main attributes of an address with \b this space
 /// and a size. The caller provides the \e offset and \e size,
 /// and other details about this particular space are filled in.
-/// \param s is the stream to write to
+/// \param encoder is the stream encoder
 /// \param offset is the offset of the address
 /// \param size is the size of the memory location
-void AddrSpace::saveXmlAttributes(ostream &s,uintb offset,int4 size) const
+void AddrSpace::encodeAttributes(Encoder &encoder,uintb offset,int4 size) const
 
 {
-  a_v(s,"space",getName());	// Just append the proper attributes
-  s << ' ' << "offset=\"";
-  printOffset(s,offset);
-  s << "\"";
-  a_v_i(s,"size",size);
+  encoder.writeSpace(ATTRIB_SPACE, this);
+  encoder.writeUnsignedInteger(ATTRIB_OFFSET, offset);
+  encoder.writeSignedInteger(ATTRIB_SIZE, size);
 }
 
-/// For an XML tag describing an address in this space, this routine
-/// recovers the offset and possibly the size described by the tag
-/// \param el is the XML address tag
+/// For an open element describing an address in \b this space, this routine
+/// recovers the offset and possibly the size described by the element
+/// \param decoder is the stream decoder
 /// \param size is a reference where the recovered size should be stored
 /// \return the recovered offset
-uintb AddrSpace::restoreXmlAttributes(const Element *el,uint4 &size) const
+uintb AddrSpace::decodeAttributes(Decoder &decoder,uint4 &size) const
 
 {
   uintb offset;
-  int4 num = el->getNumAttributes();
   bool foundoffset = false;
-  for(int4 i=0;i<num;++i) {
-    if (el->getAttributeName(i)=="offset") {
+  for(;;) {
+    uint4 attribId = decoder.getNextAttributeId();
+    if (attribId == 0) break;
+    if (attribId == ATTRIB_OFFSET) {
       foundoffset = true;
-      istringstream s1(el->getAttributeValue(i));
-      s1.unsetf(ios::dec | ios::hex | ios::oct);
-      s1 >> offset;
+      offset = decoder.readUnsignedInteger();
     }
-    else if (el->getAttributeName(i)=="size") {
-      istringstream s2(el->getAttributeValue(i));
-      s2.unsetf(ios::dec | ios::hex | ios::oct);
-      s2 >> size;
+    else if (attribId == ATTRIB_SIZE) {
+      size = decoder.readSignedInteger();
     }
   }
   if (!foundoffset)
@@ -281,7 +289,7 @@ uintb AddrSpace::read(const string &s,int4 &size) const
 }
 
 /// Write a tag fully describing the details of this space
-/// suitable for later recovery via restoreXml.
+/// suitable for later recovery via decode.
 /// \param s is the stream being written
 void AddrSpace::saveXml(ostream &s) const
 
@@ -291,56 +299,36 @@ void AddrSpace::saveXml(ostream &s) const
   s << "/>\n";
 }
 
-/// Walk a parsed XML tag and recover all the properties defining
+/// Walk attributes of the current element and recover all the properties defining
 /// this space.  The processor translator, \e trans, and the
 /// \e type must already be filled in.
-/// \param el is the parsed XML tag
-void AddrSpace::restoreXml(const Element *el)
+/// \param decoder is the stream decoder
+void AddrSpace::decodeBasicAttributes(Decoder &decoder)
 
 {
-  int4 numAttribs = el->getNumAttributes();
   deadcodedelay = -1;
-  for (int4 i=0; i < numAttribs; i++) {
-    string attrName = el->getAttributeName(i);
-    string attrValue = el->getAttributeValue(i);
-    
-    if (attrName == "name") {
-      name = attrValue;
+  for (;;) {
+    uint4 attribId = decoder.getNextAttributeId();
+    if (attribId == 0) break;
+    if (attribId == ATTRIB_NAME) {
+      name = decoder.readString();
     }
-    if (attrName == "index")
-      {
-	istringstream s1(attrValue);
-	s1.unsetf(ios::dec | ios::hex | ios::oct);
-	s1 >> index;
-      }
-    if (attrName == "size")
-      {
-	istringstream s1(attrValue);
-	s1.unsetf(ios::dec | ios::hex | ios::oct);
-	s1 >> addressSize;
-      }
-    if (attrName == "wordsize")
-      {
-	istringstream s1(attrValue);
-	s1.unsetf(ios::dec | ios::hex | ios::oct);
-	s1 >> wordsize;
-      }
-    if (attrName == "bigendian") {
-      if (xml_readbool(attrValue))
+    if (attribId == ATTRIB_INDEX)
+      index = decoder.readSignedInteger();
+    else if (attribId == ATTRIB_SIZE)
+      addressSize = decoder.readSignedInteger();
+    else if (attribId == ATTRIB_WORDSIZE)
+      wordsize = decoder.readUnsignedInteger();
+    else if (attribId == ATTRIB_BIGENDIAN) {
+      if (decoder.readBool())
 	flags |= big_endian;
     }
-    if (attrName == "delay") {
-      istringstream s1(attrValue);
-      s1.unsetf(ios::dec | ios::hex | ios::oct);
-      s1 >> delay;
-    }
-    if (attrName == "deadcodedelay") {
-      istringstream s1(attrValue);
-      s1.unsetf(ios::dec | ios::hex | ios::oct);
-      s1 >> deadcodedelay;
-    }      
-    if (attrName == "physical") {
-      if (xml_readbool(attrValue))
+    else if (attribId == ATTRIB_DELAY)
+      delay = decoder.readSignedInteger();
+    else if (attribId == ATTRIB_DEADCODEDELAY)
+      deadcodedelay = decoder.readSignedInteger();
+    else if (attribId == ATTRIB_PHYSICAL) {
+      if (decoder.readBool())
 	flags |= hasphysical;
     }
     
@@ -350,16 +338,25 @@ void AddrSpace::restoreXml(const Element *el)
   calcScaleMask();
 }
 
+void AddrSpace::decode(Decoder &decoder)
+
+{
+  uint4 elemId = decoder.openElement();		// Multiple tags: <space>, <space_other>, <space_unique>
+  decodeBasicAttributes(decoder);
+  decoder.closeElement(elemId);
+}
+
+const string ConstantSpace::NAME = "const";
+
+const int4 ConstantSpace::INDEX = 0;
+
 /// This constructs the unique constant space
 /// By convention, the name is always "const" and the index
 /// is always 0.
 /// \param m is the associated address space manager
 /// \param t is the associated processor translator
-/// \param nm is the name
-/// \param ind is the integer identifier
-ConstantSpace::ConstantSpace(AddrSpaceManager *m,const Translate *t,
-			     const string &nm,int4 ind)
-  : AddrSpace(m,t,IPTR_CONSTANT,nm,sizeof(uintb),1,ind,0,0)
+ConstantSpace::ConstantSpace(AddrSpaceManager *m,const Translate *t)
+  : AddrSpace(m,t,IPTR_CONSTANT,NAME,sizeof(uintb),1,INDEX,0,0)
 {
   clearFlags(heritaged|does_deadcode|big_endian);
   if (HOST_ENDIAN==1)		// Endianness always matches host
@@ -383,23 +380,25 @@ void ConstantSpace::saveXml(ostream &s) const
 }
 
 /// As the ConstantSpace is never saved, it should never get
-/// restored either.
-void ConstantSpace::restoreXml(const Element *el)
+/// decoded either.
+void ConstantSpace::decode(Decoder &decoder)
 
 {
-  throw LowlevelError("Should never restore the constant space from XML");
+  throw LowlevelError("Should never decode the constant space");
 }
 
+const string OtherSpace::NAME = "OTHER";
+
+const int4 OtherSpace::INDEX = 1;
+
 /// Construct the \b other space, which is automatically constructed
 /// by the compiler, and is only constructed once.  The name should
 /// always by \b OTHER.
 /// \param m is the associated address space manager
 /// \param t is the associated processor translator
-/// \param nm is the name of the space
 /// \param ind is the integer identifier
-OtherSpace::OtherSpace(AddrSpaceManager *m,const Translate *t,
-		       const string &nm,int4 ind)
-  : AddrSpace(m,t,IPTR_PROCESSOR,nm,sizeof(uintb),1,ind,0,0)
+OtherSpace::OtherSpace(AddrSpaceManager *m,const Translate *t,int4 ind)
+  : AddrSpace(m,t,IPTR_PROCESSOR,NAME,sizeof(uintb),1,INDEX,0,0)
 {
   clearFlags(heritaged|does_deadcode);
   setFlags(is_otherspace);
@@ -426,17 +425,19 @@ void OtherSpace::saveXml(ostream &s) const
   s << "/>\n";
 }
 
+const string UniqueSpace::NAME = "unique";
+
+const uint4 UniqueSpace::SIZE = 4;
+
 /// This is the constructor for the \b unique space, which is
 /// automatically constructed by the analysis engine, and
 /// constructed only once.  The name should always be \b unique.
 /// \param m is the associated address space manager
 /// \param t is the associated processor translator
-/// \param nm is the name of the space
 /// \param ind is the integer identifier
 /// \param fl are attribute flags (currently unused)
-UniqueSpace::UniqueSpace(AddrSpaceManager *m,const Translate *t,const string &nm,
-			 int4 ind,uint4 fl)
-  : AddrSpace(m,t,IPTR_INTERNAL,nm,sizeof(uintm),1,ind,fl,0)
+UniqueSpace::UniqueSpace(AddrSpaceManager *m,const Translate *t,int4 ind,uint4 fl)
+  : AddrSpace(m,t,IPTR_INTERNAL,NAME,SIZE,1,ind,fl,0)
 {
   setFlags(hasphysical);
 }
@@ -455,105 +456,91 @@ void UniqueSpace::saveXml(ostream &s) const
   s << "/>\n";
 }
 
+const string JoinSpace::NAME = "join";
+
 /// This is the constructor for the \b join space, which is automatically constructed by the
 /// analysis engine, and constructed only once. The name should always be \b join.
 /// \param m is the associated address space manager
 /// \param t is the associated processor translator
-/// \param nm is the name of the space
 /// \param ind is the integer identifier
-JoinSpace::JoinSpace(AddrSpaceManager *m,const Translate *t,const string &nm,int4 ind)
-  : AddrSpace(m,t,IPTR_JOIN,nm,sizeof(uintm),1,ind,0,0)
+JoinSpace::JoinSpace(AddrSpaceManager *m,const Translate *t,int4 ind)
+  : AddrSpace(m,t,IPTR_JOIN,NAME,sizeof(uintm),1,ind,0,0)
 {
   // This is a virtual space
   // setFlags(hasphysical);
   clearFlags(heritaged); // This space is never heritaged, but does dead-code analysis
 }
 
-/// Save a join address to the stream as XML.  This method in the interface only
-/// outputs XML attributes for a single tag, so we are forced to encode what should probably
-/// be recursive tags into an attribute
-/// \param s is the stream being written to
+/// Encode a \e join address to the stream.  This method in the interface only
+/// outputs attributes for a single element, so we are forced to encode what should probably
+/// be recursive elements into an attribute.
+/// \param encoder is the stream encoder
 /// \param offset is the offset within the address space to encode
-void JoinSpace::saveXmlAttributes(ostream &s,uintb offset) const
+void JoinSpace::encodeAttributes(Encoder &encoder,uintb offset) const
 
 {
+  static AttributeId *pieceArray[] = { &ATTRIB_PIECE1, &ATTRIB_PIECE2, &ATTRIB_PIECE3, &ATTRIB_PIECE4,
+	&ATTRIB_PIECE5, &ATTRIB_PIECE6, &ATTRIB_PIECE7, &ATTRIB_PIECE8, &ATTRIB_PIECE9 };
   JoinRecord *rec = getManager()->findJoin(offset); // Record must already exist
-  a_v(s,"space",getName());
+  encoder.writeSpace(ATTRIB_SPACE, this);
   int4 num = rec->numPieces();
+  if (num >= 8)
+    throw LowlevelError("Cannot encode more than 8 pieces");
   for(int4 i=0;i<num;++i) {
     const VarnodeData &vdata( rec->getPiece(i) );
     ostringstream t;
-    t << " piece" << dec << (i+1) << "=\"";
+    AttributeId *attribId = pieceArray[i];
     t << vdata.space->getName() << ":0x";
-    t << hex << vdata.offset << ':' << dec << vdata.size << '\"';
-    s << t.str();
+    t << hex << vdata.offset << ':' << dec << vdata.size;
+    encoder.writeString(*attribId, t.str());
   }
   if (num == 1)
-    a_v_i(s,"logicalsize",rec->getUnified().size);
-
+    encoder.writeUnsignedInteger(ATTRIB_LOGICALSIZE, rec->getUnified().size);
 }
 
-/// Save a join address to the stream as XML.  This method in the interface only
-/// outputs XML attributes for a single tag, so we are forced to encode what should probably
-/// be recursive tags into an attribute
-/// \param s is the stream being written to
+/// Encode a \e join address to the stream.  This method in the interface only
+/// outputs attributes for a single element, so we are forced to encode what should probably
+/// be recursive elements into an attribute.
+/// \param encoder is the stream encoder
 /// \param offset is the offset within the address space to encode
 /// \param size is the size of the memory location being encoded
-void JoinSpace::saveXmlAttributes(ostream &s,uintb offset,int4 size) const
+void JoinSpace::encodeAttributes(Encoder &encoder,uintb offset,int4 size) const
 
 {
-  JoinRecord *rec = getManager()->findJoin(offset); // Record must already exist
-  a_v(s,"space",getName());
-  int4 num = rec->numPieces();
-  int4 count = 0;
-  for(int4 i=0;i<num;++i) {
-    const VarnodeData &vdata( rec->getPiece(i) );
-    ostringstream t;
-    t << " piece" << dec << (i+1) << "=\"";
-    t << vdata.space->getName() << ":0x";
-    t << hex << vdata.offset << ':' << dec << vdata.size << '\"';
-    count += vdata.size;
-    s << t.str();
-  }
-  if (num == 1)
-    a_v_i(s,"logicalsize",rec->getUnified().size);
-  if ((count != size)&&(num>1))
-    throw LowlevelError("size attribute in join tag does not match size of pieces");
+  encodeAttributes(encoder,offset);	// Ignore size
 }
 
-/// Restore a join address from an XML tag.  Pieces of the join are encoded as a sequence
-/// of tag attributes.  The Translate::findAddJoin method is used to construct a logical
+/// Parse a join address the current element.  Pieces of the join are encoded as a sequence
+/// of attributes.  The Translate::findAddJoin method is used to construct a logical
 /// address within the join space.
-/// \param el is the parsed XML element to extract the address from
+/// \param decoder is the stream decoder
 /// \param size is a reference to be filled in as the size encoded by the tag
 /// \return the offset of the final address encoded by the tag
-uintb JoinSpace::restoreXmlAttributes(const Element *el,uint4 &size) const
+uintb JoinSpace::decodeAttributes(Decoder &decoder,uint4 &size) const
 
 {
   vector<VarnodeData> pieces;
-  int4 numAttribs = el->getNumAttributes();
   uint4 sizesum = 0;
   uint4 logicalsize = 0;
-  for(int4 i=0;i<numAttribs;++i) {
-    string attrName = el->getAttributeName(i);
-    if (0!=attrName.compare(0,5,"piece")) {
-      if (attrName == "logicalsize") {
-	istringstream s3(el->getAttributeValue(i));
-	s3.unsetf(ios::dec | ios::hex | ios::oct);
-	s3 >> logicalsize;
-      }
+  for(;;) {
+    uint4 attribId = decoder.getNextAttributeId();
+    if (attribId == 0) break;
+    if (attribId == ATTRIB_LOGICALSIZE) {
+      logicalsize = decoder.readUnsignedInteger();
       continue;
     }
-    int4 pos = (int4)(attrName[5] - '1');
+    if (attribId < ATTRIB_PIECE1.getId() || attribId > ATTRIB_PIECE9.getId())
+      continue;
+    int4 pos = (int4)(attribId - ATTRIB_PIECE1.getId());
     while(pieces.size() <= pos)
-      pieces.push_back(VarnodeData());
+      pieces.emplace_back();
     VarnodeData &vdat( pieces[pos] );
 
-    string attrVal = el->getAttributeValue(i);
+    string attrVal = decoder.readString();
     string::size_type offpos = attrVal.find(':');
     if (offpos == string::npos) {
-      const Translate *trans = getTrans();
-      const VarnodeData &point(trans->getRegister(attrVal));
+      const Translate *tr = getTrans();
+      const VarnodeData &point(tr->getRegister(attrVal));
       vdat = point;
     }
     else {
@@ -604,7 +591,7 @@ uintb JoinSpace::read(const string &s,int4 &size) const
   int4 szsum = 0;
   int4 i=0;
   while(i < s.size()) {
-    pieces.push_back(VarnodeData()); // Prepare to read next VarnodeData
+    pieces.emplace_back();	// Prepare to read next VarnodeData
     string token;
     while((i<s.size())&&(s[i]!=',')) {
       token += s[i];
@@ -615,8 +602,8 @@ uintb JoinSpace::read(const string &s,int4 &size) const
       pieces.back() = getTrans()->getRegister(token);
     }
     catch(LowlevelError &err) {	// Name doesn't exist
-      char shortcut = token[0];
-      AddrSpace *spc = getManager()->getSpaceByShortcut(shortcut);
+      char tryShortcut = token[0];
+      AddrSpace *spc = getManager()->getSpaceByShortcut(tryShortcut);
       if (spc == (AddrSpace *)0)
 	throw LowlevelError("Could not parse join string");
 
@@ -638,10 +625,10 @@ void JoinSpace::saveXml(ostream &s) const
   throw LowlevelError("Should never save join space to XML");
 }
 
-void JoinSpace::restoreXml(const Element *el)
+void JoinSpace::decode(Decoder &decoder)
 
 {
-  throw LowlevelError("Should never restore join space from XML");
+  throw LowlevelError("Should never decode join space");
 }
 
 /// \param m is the address space manager
@@ -653,13 +640,6 @@ OverlaySpace::OverlaySpace(AddrSpaceManager *m,const Translate *t)
   setFlags(overlay);
 }
 
-/// \return the base address space
-AddrSpace *OverlaySpace::getBaseSpace(void) const
-
-{
-  return baseSpace;
-}
-
 void OverlaySpace::saveXml(ostream &s) const
 
 {
@@ -670,18 +650,15 @@ void OverlaySpace::saveXml(ostream &s) const
   s << "/>\n";
 }
 
-void OverlaySpace::restoreXml(const Element *el)
+void OverlaySpace::decode(Decoder &decoder)
 
 {
-  name = el->getAttributeValue("name");
-  istringstream s1(el->getAttributeValue("index"));
-  s1.unsetf(ios::dec | ios::hex | ios::oct);
-  s1 >> index;
+  uint4 elemId = decoder.openElement(ELEM_SPACE_OVERLAY);
+  name = decoder.readString(ATTRIB_NAME);
+  index = decoder.readSignedInteger(ATTRIB_INDEX);
   
-  string basename = el->getAttributeValue("base");
-  baseSpace = getManager()->getSpaceByName(basename);
-  if (baseSpace == (AddrSpace *)0)
-    throw LowlevelError("Base space does not exist for overlay space: "+name);
+  baseSpace = decoder.readSpace(ATTRIB_BASE);
+  decoder.closeElement(elemId);
   addressSize = baseSpace->getAddrSize();
   wordsize = baseSpace->getWordSize();
   delay = baseSpace->getDelay();
